package ru.esc.btqckcmd.service;

import java.util.ArrayList;
import java.util.Set;

import android.bluetooth.BluetoothAdapter;
import android.bluetooth.BluetoothDevice;
import android.content.BroadcastReceiver;
import android.content.Context;
import android.content.Intent;
import android.content.IntentFilter;
import android.content.SharedPreferences;
import android.preference.PreferenceManager;

import ru.esc.btqckcmd.utils.MLog;

/**
 * Used to discover remote bluetooth device with specified name.
 */
public class DiscoverDevice {

	/**
	 * Remote device name.
	 */
	private static final String REMOTE_DEVICE_NAME = "linvor";

	/**
	 * Application {@link Context}.
	 */
	private Context mContext = null;

	/**
	 * Handler to send events to.
	 */
	private IDiscoverDevice mHandler = null;

	/**
	 * {@link BluetoothAdapter} used to start scan.
	 */
	private BluetoothAdapter mBluetoothAdapter = null;

    private String mRemoteDeviceDynamicName = null;

    private ArrayList<String> mDiscoveredDevices = null;

    /**
	 * Default constructor. Init {@link #mBluetoothAdapter}, {@link #mContext},
	 * {@link #mHandler}. Register receivers: {@link #mDeviceFoundReceiver},
	 * {@link #mDiscoveryStartedReceiver}, {@link #mDiscoveryFinishedReceiver}.
	 */
	public DiscoverDevice(Context context, IDiscoverDevice handler) {
		mContext = context;
		mHandler = handler;

		// init BluetoothAdapter object
		mBluetoothAdapter = BluetoothAdapter.getDefaultAdapter();

		if (mDiscoveredDevices != null) {
		    mDiscoveredDevices.clear();
		} else {
		    mDiscoveredDevices = new ArrayList<String>();
		}

		MLog.i("Register receivers");
		// Register the BroadcastReceiver
		mContext.registerReceiver(mDeviceFoundReceiver, new IntentFilter(BluetoothDevice.ACTION_FOUND));
		mContext.registerReceiver(mDiscoveryStartedReceiver, new IntentFilter(BluetoothAdapter.ACTION_DISCOVERY_STARTED));
		mContext.registerReceiver(mDiscoveryFinishedReceiver, new IntentFilter(BluetoothAdapter.ACTION_DISCOVERY_FINISHED));

		// SharedPreferences prefs = mContext.getSharedPreferences("general_preferences", Context.MODE_PRIVATE);
		SharedPreferences prefs = PreferenceManager.getDefaultSharedPreferences(mContext);
		mRemoteDeviceDynamicName = prefs.getString("remote_device_name", REMOTE_DEVICE_NAME);
        MLog.i("Got name from prefs: " + mRemoteDeviceDynamicName);
	}

	/**
	 * Start devices scan.
	 */
	public void startDiscovering() {
        mRemoteDeviceDynamicName = PreferenceManager.getDefaultSharedPreferences(mContext)
                .getString("remote_device_name", REMOTE_DEVICE_NAME);
		if (!deviceIsPaired()) {
			boolean result = mBluetoothAdapter.startDiscovery();
			MLog.i("Start discover device: " + result);
		}
	}

    public void startDiscoveringOnly() {
        mRemoteDeviceDynamicName = PreferenceManager.getDefaultSharedPreferences(mContext)
                .getString("remote_device_name", REMOTE_DEVICE_NAME);
        boolean result = mBluetoothAdapter.startDiscovery();
        MLog.i("Start discover device: " + result);
    }

    /**
	 * Device exists in paired devices list.
	 */
	private boolean deviceIsPaired() {
		MLog.i("Check all PAIRED devices.");
		Set<BluetoothDevice> pairedDevices = mBluetoothAdapter.getBondedDevices();
		// If there are paired devices
		if (pairedDevices.size() > 0) {
			// Loop through paired devices
			for (BluetoothDevice device : pairedDevices) {
				MLog.i("NAME: " + device.getName() + "; ADDR: " + device.getAddress());
				if (device.getName().equals(mRemoteDeviceDynamicName)) {
					mHandler.onDeviceDescovered(device);
					return true;
				}
			}
		} else {
			MLog.i("No PAIRED devices found.");
		}

		MLog.i("Device \"" + mRemoteDeviceDynamicName + "\" is NOT PAIRED");
		return false;
	}

    /**
     * Check if the device already exists in paired devices list.
     */
    private boolean isDevicePaired() {
        MLog.i("Check all PAIRED devices.");
        Set<BluetoothDevice> pairedDevices = mBluetoothAdapter.getBondedDevices();
        // If there are paired devices
        if (pairedDevices.size() > 0) {
            // Loop through paired devices
            for (BluetoothDevice device : pairedDevices) {
                MLog.i("NAME: " + device.getName() + "; ADDR: " + device.getAddress());
                if (device.getName().equals(mRemoteDeviceDynamicName)) {
                    return true;
                }
            }
        } else {
            MLog.i("No PAIRED devices found.");
        }

        MLog.i("Device \"" + mRemoteDeviceDynamicName + "\" is NOT PAIRED");
        return false;
    }

    public ArrayList<String> getDiscoveredList() {
	    return mDiscoveredDevices;
	}

	/**
	 * Create a BroadcastReceiver for ACTION_DISCOVERY_STARTED
	 */
	private final BroadcastReceiver mDiscoveryStartedReceiver = new BroadcastReceiver() {
		public void onReceive(Context context, Intent intent) {
			mHandler.onScanStarted();
		}
	};

	/**
	 * Create a BroadcastReceiver for ACTION_FOUND
	 */
	private final BroadcastReceiver mDeviceFoundReceiver = new BroadcastReceiver() {
		public void onReceive(Context context, Intent intent) {
			String action = intent.getAction();
			// When discovery finds a device
			if (BluetoothDevice.ACTION_FOUND.equals(action)) {
				// Get the BluetoothDevice object from the Intent
				BluetoothDevice device = intent.getParcelableExtra(BluetoothDevice.EXTRA_DEVICE);
				// Add the name and address to an array adapter to show in a
				// ListView
				mDiscoveredDevices.add(device.getName());
                MLog.i("Added to list: " + device.getName());
				MLog.i("NAME: " + device.getName() + "; ADDR: " + device.getAddress());

				if (device.getName() != null) {
					if (device.getName().equals(mRemoteDeviceDynamicName)) {
						mBluetoothAdapter.cancelDiscovery();
						mHandler.onDeviceDescovered(device);
					}
				}
			}
		}
	};

	/**
	 * Create a BroadcastReceiver for ACTION_DISCOVERY_FINISHED
	 */
	private final BroadcastReceiver mDiscoveryFinishedReceiver = new BroadcastReceiver() {
		public void onReceive(Context context, Intent intent) {
			mHandler.onScanFinished();
		}
	};

	/**
	 * Call when service will finish soon.
	 */
	public void onDestroy() {
		MLog.i("DiscoverDevice onDestroy");
		// unregister receivers
		mContext.unregisterReceiver(mDeviceFoundReceiver);
		mContext.unregisterReceiver(mDiscoveryStartedReceiver);
		mContext.unregisterReceiver(mDiscoveryFinishedReceiver);
	}
}
