package ru.esc.btqckcmd.service;

import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.OutputStream;
import java.io.OutputStreamWriter;
import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.util.UUID;

import android.bluetooth.BluetoothDevice;
import android.bluetooth.BluetoothSocket;

import ru.esc.btqckcmd.utils.MLog;

/**
 * Used to interact with remote device when it is connected.
 */
public class RemoteEngine {

	/**
	 * Iterations count to wait for auto reconnect when remote device is
	 * disconnected.
	 */
	private static final int CONNECTION_WAIT_ITERATIONS_COUNT = 5;

	/**
	 * {@link BluetoothDevice} object.
	 */
	private BluetoothDevice mBluetoothDevice = null;
	/**
	 * {@link BluetoothSocket} object.
	 */
	private BluetoothSocket mBtSocket = null;

	/**
	 * Handler for events from current class.
	 */
	private IRemoteEngine mHandler = null;

	/**
	 * {@link Thread} used to establish connection with remote device if it was
	 * previously found by {@link DiscoverDevice}.
	 */
	private Thread mConnectThread = null;

	/**
	 * {@link Thread} used to listen for incoming requests from remote device.
	 */
	private Thread mListeningThread = null;

	/**
	 * Indicates if {@link #mListeningThread} is running.
	 */
	private boolean mListeningThreadRunning = false;

	/**
	 * Class constructor.
	 * 
	 * @param handler
	 *            - handler to send events to.
	 */
	public RemoteEngine(IRemoteEngine handler) {
		mHandler = handler;
	}

	/**
	 * Init {@link #mBluetoothDevice} object.
	 * 
	 * @param device
	 *            - {@link BluetoothDevice} found by {@link DiscoverDevice}
	 *            object.
	 */
	public void setBluetoothDevice(BluetoothDevice device) {
		mBluetoothDevice = device;
	}

	/**
	 * Connects to remote device using SDP.
	 */
	public void connectToService() {
		mConnectThread = new Thread() {
			@Override
			public void run() {
				super.run();
				try {
					if (mBluetoothDevice == null) {
						mHandler.onConnectionError("No device set");
						return;
					}
					mBtSocket = mBluetoothDevice.createRfcommSocketToServiceRecord(UUID
							.fromString("00001101-0000-1000-8000-00805F9B34FB"));
					mBtSocket.connect();
					mHandler.onConnected();
					if (!mListeningThreadRunning) {
						startListeningIncomingData();
					}
				} catch (IOException e) {
					MLog.p(e);
					try {
						Method m = mBluetoothDevice.getClass().getMethod("createRfcommSocket", new Class[] { int.class });
						mBtSocket = (BluetoothSocket) m.invoke(mBluetoothDevice, 1);
						mBtSocket.connect();
						mHandler.onConnected();
						if (!mListeningThreadRunning) {
							startListeningIncomingData();
						}
					} catch (IOException e1) {
						MLog.p(e1);
						mHandler.onConnectionError(e1.toString());
					} catch (SecurityException e1) {
						MLog.p(e1);
						mHandler.onConnectionError(e1.toString());
					} catch (NoSuchMethodException e1) {
						MLog.p(e1);
						mHandler.onConnectionError(e1.toString());
					} catch (IllegalArgumentException e1) {
						MLog.p(e1);
						mHandler.onConnectionError(e1.toString());
					} catch (IllegalAccessException e1) {
						MLog.p(e1);
						mHandler.onConnectionError(e1.toString());
					} catch (InvocationTargetException e1) {
						MLog.p(e1);
						mHandler.onConnectionError(e1.toString());
					}

				}
			}

			@Override
			public void interrupt() {
				super.interrupt();
				if (mBtSocket != null) {
					try {
						mBtSocket.close();
					} catch (IOException e) {
						MLog.p(e);
					}
				}
			}
		};

		mConnectThread.start();
	}

	/**
	 * Start incoming request listener {@link Thread}.
	 */
	private void startListeningIncomingData() {
		if (mListeningThread != null) {
			Thread th = mListeningThread;
			mListeningThread = null;
			th.interrupt();
		}

		mListeningThread = new Thread() {

			private boolean mInterrupt = false;

			@Override
			public void run() {
				super.run();

				mListeningThreadRunning = true;

				if (mBtSocket == null) {
					MLog.e("Socket is null");
					mListeningThreadRunning = false;
					return;
				}

				String readLine = null;
				BufferedReader in;
				try {
					in = new BufferedReader(new InputStreamReader(mBtSocket.getInputStream()));
				} catch (IOException e) {
					MLog.p(e);
					mListeningThreadRunning = false;
					return;
				}

				int waitConnectionCnt = CONNECTION_WAIT_ITERATIONS_COUNT;

				while (!mInterrupt) {
					MLog.i("Wait for incoming data");
					try {
						if ((readLine = in.readLine()) != null) {
							if (readLine.length() != 0) {
								MLog.i("Read line: " + readLine);
								if (readLine.contains("ERROR")) {
									mHandler.onCommandError(readLine);
								} else {
									mHandler.onIncomingMessage(readLine);
								}
							}
						}
					} catch (IOException e) {
						if (waitConnectionCnt == 0) {
							MLog.p(e);
							mHandler.onDisconnected();
							break;
						}

						waitConnectionCnt--;
						try {
							Thread.sleep(1000);
						} catch (InterruptedException e1) {
							mInterrupt = true;
						}
					}
				}
			}

			@Override
			public void interrupt() {
				super.interrupt();
				MLog.i("Interrupting listening thread");
				mInterrupt = true;
				mListeningThreadRunning = false;
			}
		};

		mListeningThread.start();
	}

	/**
	 * Send command to remote device.
	 * 
	 * @param command
	 *            - command to send.
	 * @return {@code true} if command was sent.
	 */
	public boolean sendCommand(int command) {
		if (mBtSocket == null) {
			MLog.e("Bluetooth socket is null; can't send command");
			return false;
		}

		if (!mListeningThreadRunning) {
			startListeningIncomingData();
		}

		if (command < 0 || command > 255)
			throw new RuntimeException("Invalid command: " + command
					+ ". The value must be between 0 and 255");

		try {
			// BufferedWriter out = new BufferedWriter(new OutputStreamWriter(mBtSocket.getOutputStream()));
			OutputStream out = mBtSocket.getOutputStream();
			out.write(command);
			out.flush();
			MLog.i("command sent");
			return true;
		} catch (IOException e) {
			MLog.p(e);
			return false;
		}
	}

	/**
	 * Call when service will stop soon.
	 */
	public void onDestroy() {
		if (mConnectThread != null) {
			MLog.i("Interrupt connect thread");
			Thread th = mConnectThread;
			mConnectThread = null;
			th.interrupt();
		}

		if (mBtSocket != null) {
			MLog.i("Close bluetooth socket");
			try {
				mBtSocket.close();
			} catch (IOException e) {
				MLog.p(e);
			}
		}

		if (mListeningThread != null) {
			MLog.i("Interrupt listening thread");
			Thread th = mListeningThread;
			mListeningThread = null;
			th.interrupt();
		}
	}
}
