package ru.esc.btqckcmd.service;

import android.app.Service;
import android.bluetooth.BluetoothDevice;
import android.content.Intent;
import android.os.Binder;
import android.os.Handler;
import android.os.IBinder;

import ru.esc.btqckcmd.utils.MLog;

import java.util.ArrayList;

public class RemoteService extends Service implements IDiscoverDevice, IRemoteEngine {

	/**
	 * Service inactivity time in milliseconds. Used to stop service if no
	 * operations made by service during this period.
	 */
	private static final long SERVICE_INACTIVITY_TIME = 30000;

	/**
	 * {@link IBinder} object return to client when it binds service.
	 */
	private IBinder mBinder = new RemoteServiceBinder();

	/**
	 * Handler used to check if service is inactive for time period set in
	 * {@link #SERVICE_INACTIVITY_TIME}
	 */
	private Handler mActivityCheckerHandler = null;

	/**
	 * Handler object used in service to send events to.
	 */
	private IRemoteService mHandler = null;

	/**
	 * {@link DiscoverDevice} object. Used to discover remote device.
	 */
	private DiscoverDevice mDiscoverDevice = null;

	/**
	 * {@link RemoteEngine} object. Used to intaract with remote device.
	 */
	private RemoteEngine mRemoteEngine = null;

	/**
	 * Indicates if device discovered during scan.
	 */
	private boolean mDeviceDiscovered = false;
	/**
	 * Indicates if device connected.
	 */
	private boolean mConnected = false;
	/**
	 * Indicates if device is connecting.
	 */
	private boolean mConnecting = false;

	/**
	 * @see android.app.Service#onCreate()
	 */
	@Override
	public void onCreate() {
		super.onCreate();

		// init DiscoverDevice object
		mDiscoverDevice = new DiscoverDevice(RemoteService.this, RemoteService.this);

		// init RemoteEngine object
		mRemoteEngine = new RemoteEngine(RemoteService.this);

		mActivityCheckerHandler = new Handler();
	}

	/**
	 * @see android.app.Service#onStart(android.content.Intent, int)
	 */
	@Override
	public void onStart(Intent intent, int startId) {
		MLog.i("Service Start");
		mActivityCheckerHandler.removeCallbacks(mActivityCheckerRunnable);
		super.onStart(intent, startId);
	}

	/**
	 * @see android.app.Service#onStartCommand(android.content.Intent, int, int)
	 */
	@Override
	public int onStartCommand(Intent intent, int flags, int startId) {
		MLog.i("Service Start Command");
		mActivityCheckerHandler.removeCallbacks(mActivityCheckerRunnable);
		return START_STICKY;
	}

	/**
	 * @see android.app.Service#onBind(android.content.Intent)
	 */
	@Override
	public IBinder onBind(Intent intent) {
		MLog.i("Service bind");
		mActivityCheckerHandler.removeCallbacks(mActivityCheckerRunnable);
		return mBinder;
	}

	/**
	 * @see android.app.Service#onUnbind(android.content.Intent)
	 */
	@Override
	public boolean onUnbind(Intent intent) {
		MLog.i("Service unbind");
		mActivityCheckerHandler.postDelayed(mActivityCheckerRunnable, SERVICE_INACTIVITY_TIME);
		return true;
	}

	/**
	 * @see android.app.Service#onDestroy()
	 */
	@Override
	public void onDestroy() {
		super.onDestroy();
		MLog.i("Service onDestroy()");
		mRemoteEngine.onDestroy();
		mDiscoverDevice.onDestroy();
	}

	/**
	 * Used to by clients to get current {@link RemoteService} object.
	 */
	public class RemoteServiceBinder extends Binder {
		public RemoteService getService() {
			return RemoteService.this;
		}
	}

	/**
	 * Set client handler for service events.
	 * 
	 * @param handler
	 *            - client handler.
	 */
	public void setHandler(IRemoteService handler) {
		mHandler = handler;
	}

	/**
	 * Starts device discovering.
	 */
	public void startDeviceDiscover() {
		mConnecting = true;
		mHandler.onScanStarted();
		mDiscoverDevice.startDiscovering();
	}

	/**
	 * @see ru.esc.btqckcmd.service.IDiscoverDevice#onScanStarted()
	 */
	public void onScanStarted() {
		mDeviceDiscovered = false;
		MLog.i("Discovery STARTED");
	}

	/**
	 * @see ru.esc.btqckcmd.service.IDiscoverDevice#onDeviceDescovered(android.bluetooth.BluetoothDevice)
	 */
	public void onDeviceDescovered(BluetoothDevice device) {
		mDeviceDiscovered = true;
		MLog.i("DISCOVERED DEVICE: NAME: " + device.getName() + "; ADDR: " + device.getAddress());
		mRemoteEngine.setBluetoothDevice(device);
		mRemoteEngine.connectToService();
	}

	/**
	 * @see ru.esc.btqckcmd.service.IDiscoverDevice#onScanFinished()
	 */
	public void onScanFinished() {
		if (!mDeviceDiscovered) {
			mConnecting = false;
            // ArrayList<String> discoveredList = mDiscoverDevice.getDiscoveredList();
            // mHandler.onDeviceNotFound(discoveredList);
            mHandler.onDeviceNotFound();
		}
		MLog.i("Discovery FINISHED");
	}

	/**
	 * @see ru.esc.btqckcmd.service.IRemoteEngine#onConnected()
	 */
	public void onConnected() {
		mConnecting = false;
		mConnected = true;
		mHandler.onDeviceConnected();

		MLog.i("CONNECTED");

	}

	/**
	 * @see ru.esc.btqckcmd.service.IRemoteEngine#onConnectionError(java.lang.String)
	 */
	public void onConnectionError(String error) {
		mConnecting = false;
		mConnected = false;
		// mDiscoverDevice.startDiscoveringOnly();
		mHandler.onDeviceNotConnected();
	}

	/**
	 * @see ru.esc.btqckcmd.service.IRemoteEngine#onDisconnected()
	 */
	public void onDisconnected() {
		mConnecting = false;
		mConnected = false;
		mHandler.onDisconnect();

		mRemoteEngine.onDestroy();
	}

	/**
	 * @see ru.esc.btqckcmd.service.IRemoteEngine#onCommandError(java.lang.String)
	 */
	public void onCommandError(String error) {
		MLog.w("Command ERROR: " + error);
	}

	/**
	 * @see ru.esc.btqckcmd.service.IRemoteEngine#onIncomingMessage(java.lang.String)
	 */
	public void onIncomingMessage(String message) {
		MLog.i("Received from device: " + message);
	}

	/**
	 * Send command to remote device.
	 * @param sendCmd
	 */
	public void sendCommand(int sendCmd) {
		if (mConnected) {
			//sendCmd += "\r\n";

			//final String lcmd = sendCmd.substring(0, sendCmd.length() - 2);

			//MLog.i("SEND " + lcmd);
			mRemoteEngine.sendCommand(sendCmd);
		} else {
			MLog.w("Can't send command: not connected");
		}
	}

	/**
	 * @return {@code true} if remote device is connected.
	 */
	public boolean isConnected() {
		return mConnected;
	}

	/**
	 * @return {@code true} if remote device is connecting now.
	 */
	public boolean isConnecting() {
		return mConnecting;
	}

	/**
	 * {@link Runnable} used to check if service is inactive for {@link #SERVICE_INACTIVITY_TIME} time.
	 */
	Runnable mActivityCheckerRunnable = new Runnable() {
		public void run() {
			MLog.i("Service was inactive for " + String.valueOf(SERVICE_INACTIVITY_TIME / 1000) + " sec. Stop service.");
			RemoteService.this.stopSelf();
		}
	};
}
